/****************************************************************************/
/*                       Pro SQL Server Internals                           */
/*      APress. 2nd Edition. ISBN-13: 978-1484219638 ISBN-10:1484219635     */
/*                                                                          */
/*                  Written by Dmitri V. Korotkevitch                       */
/*                      http://aboutsqlserver.com                           */
/*                        dk@aboutsqlserver.com                             */
/****************************************************************************/
/*                      Chapter 09. Lock Partitioning				        */
/*                 Analyzing Lock Partitioning (Session 3)                  */
/****************************************************************************/

-- You need to have 16 or more schedulers for lock partitioning to be enabled

-- You can artificially change number of cores with undocumented startup flag -P[N] 
-- [N] is number of cores. DO NOT DO THIS IN PRODUCTION!

use SQLServerInternals
go

if exists(select * from sys.tables t join sys.schemas s on t.schema_id = s.schema_id where s.name = 'dbo' and t.name = 'Numbers') drop table dbo.Numbers
go

create table dbo.Numbers
(
	Number int not null 
		constraint PK_Numbers
		primary key(Number),
	Col int null
);

;with N1(C) as (select 0 union all select 0) -- 2 rows
,N2(C) as (select 0 from N1 as t1 cross join N1 as t2) -- 4 rows
,N3(C) as (select 0 from N2 as t1 cross join N2 as t2) -- 16 rows
,N4(C) as (select 0 from N3 as t1 cross join N3 as t2) -- 256 rows
,N5(C) as (select 0 from N4 as t1 cross join N4 as t2) -- 65,536 rows
,N6(C) as (select 0 from N5 as t1 cross join N3 as t2) -- 1,048,576 rows
,Ids(Id) as (select row_number() over (order by (select null)) from N6)
insert into dbo.Numbers(Number)
	select Id from Ids;
go

alter table dbo.Numbers set (lock_escalation = disable);
go

begin tran
	update dbo.Numbers with (rowlock)
	set Col = 1;

	waitfor delay '00:00:05.000'; -- Allow counter to refresh
	
	select cntr_value 
	from sys.dm_os_performance_counters
	where 
		object_name = 'SQLServer:Memory Manager' and
		counter_name = 'Lock Memory (KB)';

	select count(*) from sys.dm_tran_locks
rollback
go
